==> rustc-1.76_1.76.0+dfsg0ubuntu1~bpo0.orig.tar.xz: rustc-1.76.0-src/compiler/rustc_middle/src/util/bug.rs <==
// These functions are used by macro expansion for bug! and span_bug!

use crate::ty::{tls, TyCtxt};
use rustc_errors::MultiSpan;
use rustc_span::Span;
use std::fmt;
use std::panic::{panic_any, Location};

#[cold]
#[inline(never)]
#[track_caller]
pub fn bug_fmt(args: fmt::Arguments<'_>) -> ! {
    // this wrapper mostly exists so I don't have to write a fully
    // qualified path of None::<Span> inside the bug!() macro definition
    opt_span_bug_fmt(None::<Span>, args, Location::caller());
}

#[cold]
#[inline(never)]
#[track_caller]
pub fn span_bug_fmt<S: Into<MultiSpan>>(span: S, args: fmt::Arguments<'_>) -> ! {
    opt_span_bug_fmt(Some(span), args, Location::caller());
}

#[track_caller]
fn opt_span_bug_fmt<S: Into<MultiSpan>>(
    span: Option<S>,
    args: fmt::Arguments<'_>,
    location: &Location<'_>,
) -> ! {
    tls::with_opt(move |tcx| {
        let msg = format!("{location}: {args}");
        match (tcx, span) {
            (Some(tcx), Some(span)) => tcx.sess.dcx().span_bug(span, msg),
            (Some(tcx), None) => tcx.sess.dcx().bug(msg),
            (None, _) => panic_any(msg),
        }
    })
}

/// A query to trigger a `span_delayed_bug`. Clearly, if one has a `tcx` one can already trigger a
/// `span_delayed_bug`, so what is the point of this? It exists to help us test `span_delayed_bug`'s
/// interactions with the query system and incremental.
pub fn trigger_span_delayed_bug(tcx: TyCtxt<'_>, key: rustc_hir::def_id::DefId) {
    tcx.sess.span_delayed_bug(
        tcx.def_span(key),
        "delayed span bug triggered by #[rustc_error(span_delayed_bug_from_inside_query)]",
    );
}

pub fn provide(providers: &mut crate::query::Providers) {
    *providers = crate::query::Providers { trigger_span_delayed_bug, ..*providers };
}

==> rustc-1.77_1.77.2+dfsg1ubuntu1~bpo0.orig.tar.xz: rustc-1.77.2-src/compiler/rustc_middle/src/util/bug.rs <==
// These functions are used by macro expansion for bug! and span_bug!

use crate::ty::{tls, TyCtxt};
use rustc_errors::MultiSpan;
use rustc_span::Span;
use std::fmt;
use std::panic::{panic_any, Location};

#[cold]
#[inline(never)]
#[track_caller]
pub fn bug_fmt(args: fmt::Arguments<'_>) -> ! {
    // this wrapper mostly exists so I don't have to write a fully
    // qualified path of None::<Span> inside the bug!() macro definition
    opt_span_bug_fmt(None::<Span>, args, Location::caller());
}

#[cold]
#[inline(never)]
#[track_caller]
pub fn span_bug_fmt<S: Into<MultiSpan>>(span: S, args: fmt::Arguments<'_>) -> ! {
    opt_span_bug_fmt(Some(span), args, Location::caller());
}

#[track_caller]
fn opt_span_bug_fmt<S: Into<MultiSpan>>(
    span: Option<S>,
    args: fmt::Arguments<'_>,
    location: &Location<'_>,
) -> ! {
    tls::with_opt(move |tcx| {
        let msg = format!("{location}: {args}");
        match (tcx, span) {
            (Some(tcx), Some(span)) => tcx.dcx().span_bug(span, msg),
            (Some(tcx), None) => tcx.dcx().bug(msg),
            (None, _) => panic_any(msg),
        }
    })
}

/// A query to trigger a `span_delayed_bug`. Clearly, if one has a `tcx` one can already trigger a
/// `span_delayed_bug`, so what is the point of this? It exists to help us test `span_delayed_bug`'s
/// interactions with the query system and incremental.
pub fn trigger_span_delayed_bug(tcx: TyCtxt<'_>, key: rustc_hir::def_id::DefId) {
    tcx.dcx().span_delayed_bug(
        tcx.def_span(key),
        "delayed span bug triggered by #[rustc_error(span_delayed_bug_from_inside_query)]",
    );
}

pub fn provide(providers: &mut crate::query::Providers) {
    *providers = crate::query::Providers { trigger_span_delayed_bug, ..*providers };
}
